@@ -1,84 +1,83 @@
-guess <- function(x) {
+guess = function(x) {
   if ("value" %chin% names(x))
     return("value")
   if ("(all)" %chin% names(x))
     return("(all)")
-  var <- names(x)[ncol(x)]
-  message("Using '", var, "' as value column. Use 'value.var' to override")
+  var = names(x)[ncol(x)]
+  messagef("Using '%s' as value column. Use 'value.var' to override", var)
   return(var)
 }
 
-dcast <- function(data, formula, fun.aggregate = NULL, ..., margins = NULL,
-      subset = NULL, fill = NULL, value.var = guess(data)) {
-  if (is.data.table(data))
-    UseMethod("dcast", data)
+dcast <- function(
+  data, formula, fun.aggregate = NULL, ..., margins = NULL,
+  subset = NULL, fill = NULL, value.var = guess(data)
+) {
+  if (is.data.table(data)) UseMethod("dcast", data)
+  # nocov start
   else {
-    # reshape2::dcast is not generic so we have to call it explicitly. See comments at the top of fmelt.R too.
-    # nocov start
+    data_name = deparse(substitute(data))
     ns = tryCatch(getNamespace("reshape2"), error=function(e)
-         stop("The dcast generic in data.table has been passed a ",class(data)[1L]," (not a data.table) but the reshape2 package is not installed to process this type. Please either install reshape2 and try again, or pass a data.table to dcast instead."))
+      stopf("The %1$s generic in data.table has been passed a %2$s, but data.table::%1$s currently only has a method for data.tables. Please confirm your input is a data.table, with setDT(%3$s) or as.data.table(%3$s). If you intend to use a method from reshape2, try installing that package first, but do note that reshape2 is superseded and is no longer actively developed.", "dcast", class(data)[1L], data_name))
+    warningf("The %1$s generic in data.table has been passed a %2$s and will attempt to redirect to the relevant reshape2 method; please note that reshape2 is superseded and is no longer actively developed, and this redirection is now deprecated. Please do this redirection yourself like reshape2::%1$s(%3$s). In the next version, this warning will become an error.", "dcast", class(data)[1L], data_name)
     ns$dcast(data, formula, fun.aggregate = fun.aggregate, ..., margins = margins,
              subset = subset, fill = fill, value.var = value.var)
-    # nocov end
   }
+  # nocov end
 }
 
-check_formula <- function(formula, varnames, valnames) {
+check_formula = function(formula, varnames, valnames) {
   if (is.character(formula)) formula = as.formula(formula)
   if (!inherits(formula, "formula") || length(formula) != 3L)
-    stop("Invalid formula. Cast formula should be of the form LHS ~ RHS, for e.g., a + b ~ c.")  # nocov; couldn't find a way to construct a test formula with length!=3L
+    stopf("Invalid formula. Cast formula should be of the form LHS ~ RHS, for e.g., a + b ~ c.")  # nocov; couldn't find a way to construct a test formula with length!=3L
   vars = all.vars(formula)
   vars = vars[!vars %chin% c(".", "...")]
   allvars = c(vars, valnames)
   if (any(allvars %chin% varnames[duplicated(varnames)]))
-    stop('data.table to cast must have unique column names')
-  ans = deparse_formula(as.list(formula)[-1L], varnames, allvars)
+    stopf('data.table to cast must have unique column names')
+  deparse_formula(as.list(formula)[-1L], varnames, allvars)
 }
 
-deparse_formula <- function(expr, varnames, allvars) {
+deparse_formula = function(expr, varnames, allvars) {
   lvars = lapply(expr, function(this) {
-    if (is.call(this)) {
-      if (this[[1L]] == quote(`+`))
-        unlist(deparse_formula(as.list(this)[-1L], varnames, allvars))
-      else this
-    } else if (is.name(this)) {
-      if (this == quote(`...`)) {
-        subvars = setdiff(varnames, allvars)
-        lapply(subvars, as.name)
-      } else this
+    if (!is.language(this)) return(NULL)
+    if (this %iscall% '+') return(unlist(deparse_formula(this[-1L], varnames, allvars)))
+    if (is.name(this) && this == quote(`...`)) {
+      subvars = setdiff(varnames, allvars)
+      return(lapply(subvars, as.name))
     }
+    this
   })
   lvars = lapply(lvars, function(x) if (length(x) && !is.list(x)) list(x) else x)
 }
 
-value_vars <- function(value.var, varnames) {
+value_vars = function(value.var, varnames) {
   if (is.character(value.var))
     value.var = list(value.var)
   value.var = lapply(value.var, unique)
   valnames = unique(unlist(value.var))
   iswrong = which(!valnames %chin% varnames)
   if (length(iswrong))
-    stop("value.var values [", paste(value.var[iswrong], collapse=", "), "] are not found in 'data'.")
+    stopf("value.var values %s are not found in 'data'.", brackify(value.var[iswrong]))
   value.var
 }
 
-aggregate_funs <- function(funs, vals, sep="_", ...) {
-  if (is.call(funs) && funs[[1L]] == "eval")
+aggregate_funs = function(funs, vals, sep="_", ...) {
+  if (funs %iscall% 'eval')
     funs = eval(funs[[2L]], parent.frame(2L), parent.frame(2L))
-  if (is.call(funs) && as.character(funs[[1L]]) %chin% c("c", "list")) {
+  if (funs %iscall% c('c', 'list')) {
     funs = lapply(as.list(funs)[-1L], function(x) {
-      if (is.call(x) && as.character(x[[1L]]) %chin% c("c", "list")) as.list(x)[-1L] else x
+      if (x %iscall% c('c', 'list')) as.list(x)[-1L] else x
     })
   } else funs = eval(funs, parent.frame(2L), parent.frame(2L))
   if(is.function(funs)) funs = list(funs) # needed for cases as shown in test#1700.1
   if (length(funs) != length(vals)) {
     if (length(vals) == 1L)
       vals = replicate(length(funs), vals)
-    else stop("When 'fun.aggregate' and 'value.var' are both lists, 'value.var' must be either of length =1 or =length(fun.aggregate).")
+    else stopf("When 'fun.aggregate' and 'value.var' are both lists, 'value.var' must be either of length =1 or =length(fun.aggregate).")
   }
   only_one_fun = length(unlist(funs)) == 1L
   dots = list(...)
-  construct_funs <- function(fun, nm, val) {
+  construct_funs = function(fun, nm, val) {
     ans = vector("list", length(fun)*length(val))
     nms = vector("character", length(ans))
     k = 1L
@@ -96,9 +95,9 @@ aggregate_funs <- function(funs, vals, sep="_", ...) {
     setattr(ans, 'names', nms)
   }
   ans = lapply(seq_along(funs), function(i) {
-    nm <- names(funs[i])
+    nm = names(funs[i])
     if (is.null(nm) || !nzchar(nm)) {
-      nm <- all.names(funs[[i]], max.names=1L, functions=TRUE)
+      nm = all.names(funs[[i]], max.names=1L, functions=TRUE)
     }
     if (!length(nm)) nm <- paste0("fun", i)
     construct_funs(funs[i], nm, vals[[i]])
@@ -106,23 +105,27 @@ aggregate_funs <- function(funs, vals, sep="_", ...) {
   as.call(c(quote(list), unlist(ans)))
 }
 
-dcast.data.table <- function(data, formula, fun.aggregate = NULL, sep = "_", ..., margins = NULL, subset = NULL, fill = NULL, drop = TRUE, value.var = guess(data), verbose = getOption("datatable.verbose")) {
-  if (!is.data.table(data)) stop("'data' must be a data.table.")
+dcast.data.table = function(data, formula, fun.aggregate = NULL, sep = "_", ..., margins = NULL, subset = NULL, fill = NULL, drop = TRUE, value.var = guess(data), verbose = getOption("datatable.verbose")) {
+  if (!is.data.table(data)) stopf("'data' must be a data.table.")
   drop = as.logical(rep(drop, length.out=2L))
-  if (anyNA(drop)) stop("'drop' must be logical TRUE/FALSE")
+  if (anyNA(drop)) stopf("'drop' must be logical TRUE/FALSE")
+  # #2980 if explicitly providing fun.aggregate=length but not a value.var,
+  #   just use the last column (as guess(data) would do) because length will be
+  #   the same on all columns
+  if (missing(value.var) && !missing(fun.aggregate) && identical(fun.aggregate, length))
+    value.var = names(data)[ncol(data)]
   lvals = value_vars(value.var, names(data))
   valnames = unique(unlist(lvals))
   lvars = check_formula(formula, names(data), valnames)
-  lvars = lapply(lvars, function(x) if (!length(x)) quote(`.`) else x)
+  lvars = lapply(lvars, function(x) if (length(x)) x else quote(`.`))
   # tired of lapply and the way it handles environments!
   allcols = c(unlist(lvars), lapply(valnames, as.name))
   dat = vector("list", length(allcols))
   for (i in seq_along(allcols)) {
     x = allcols[[i]]
-    dat[[i]] = if (identical(x, quote(`.`))) rep(".", nrow(data))
-            else eval(x, data, parent.frame())
+    dat[[i]] = if (identical(x, quote(`.`))) rep(".", nrow(data)) else eval(x, data, parent.frame())
     if (is.function(dat[[i]]))
-      stop("Column [", deparse(x), "] not found or of unknown type.")
+      stopf("Column [%s] not found or of unknown type.", deparse(x))
   }
   setattr(lvars, 'names', c("lhs", "rhs"))
   # Have to take care of duplicate names, and provide names for expression columns properly.
@@ -136,48 +139,44 @@ dcast.data.table <- function(data, formula, fun.aggregate = NULL, sep = "_", ...
   lhsnames = head(varnames, length(lvars$lhs))
   rhsnames = tail(varnames, -length(lvars$lhs))
   setattr(dat, 'names', c(varnames, valnames))
-  setDT(dat)
-  if (any(vapply_1b(as.list(dat)[varnames], is.list))) {
-    stop("Columns specified in formula can not be of type list")
+  if (any(vapply_1b(dat[varnames], is.list))) {
+    stopf("Columns specified in formula can not be of type list")
   }
-  m <- as.list(match.call()[-1L])
-  subset <- m[["subset"]][[2L]]
+  setDT(dat)
+
+  m = as.list(match.call()[-1L])
+  subset = m[["subset"]][[2L]]
   if (!is.null(subset)) {
     if (is.name(subset)) subset = as.call(list(quote(`(`), subset))
     idx = which(eval(subset, data, parent.frame())) # any advantage thro' secondary keys?
     dat = .Call(CsubsetDT, dat, idx, seq_along(dat))
   }
-  if (!nrow(dat) || !ncol(dat)) stop("Can not cast an empty data.table")
   fun.call = m[["fun.aggregate"]]
   fill.default = NULL
   if (is.null(fun.call)) {
     oo = forderv(dat, by=varnames, retGrp=TRUE)
-    if (attr(oo, 'maxgrpn') > 1L) {
-      message("Aggregate function missing, defaulting to 'length'")
+    if (attr(oo, 'maxgrpn', exact=TRUE) > 1L) {
+      messagef("Aggregate function missing, defaulting to 'length'")
       fun.call = quote(length)
     }
   }
   if (!is.null(fun.call)) {
     fun.call = aggregate_funs(fun.call, lvals, sep, ...)
-    errmsg = "Aggregating function(s) should take vector inputs and return a single value (length=1). However, function(s) returns length!=1. This value will have to be used to fill any missing combinations, and therefore must be length=1. Either override by setting the 'fill' argument explicitly or modify your function to handle this case appropriately."
+    errmsg = gettext("Aggregating function(s) should take vector inputs and return a single value (length=1). However, function(s) returns length!=1. This value will have to be used to fill any missing combinations, and therefore must be length=1. Either override by setting the 'fill' argument explicitly or modify your function to handle this case appropriately.")
     if (is.null(fill)) {
-      fill.default <- suppressWarnings(dat[0L][, eval(fun.call)])
-      # tryCatch(fill.default <- dat[0L][, eval(fun.call)], error = function(x) stop(errmsg, call.=FALSE))
-      if (nrow(fill.default) != 1L) stop(errmsg, call.=FALSE)
-    }
-    if (!any(valnames %chin% varnames)) {
-      dat = dat[, eval(fun.call), by=c(varnames)]
-    } else {
-      dat = dat[, { .SD; eval(fun.call) }, by=c(varnames), .SDcols = valnames]
+      fill.default = suppressWarnings(dat[0L][, eval(fun.call)])
+      # tryCatch(fill.default <- dat[0L][, eval(fun.call)], error = function(x) stopf(errmsg))
+      if (nrow(fill.default) != 1L) stopf(errmsg)
     }
+    dat = dat[, eval(fun.call), by=c(varnames)]
   }
-  order_ <- function(x) {
+  order_ = function(x) {
     o = forderv(x, retGrp=TRUE, sort=TRUE)
-    idx = attr(o, 'starts')
+    idx = attr(o, 'starts', exact=TRUE)
     if (!length(o)) o = seq_along(x)
     o[idx] # subsetVector retains attributes, using R's subset for now
   }
-  cj_uniq <- function(DT) {
+  cj_uniq = function(DT) {
     do.call("CJ", lapply(DT, function(x)
       if (is.factor(x)) {
         xint = seq_along(levels(x))
@@ -211,7 +210,7 @@ dcast.data.table <- function(data, formula, fun.aggregate = NULL, sep = "_", ...
       lhs = lhs_; rhs = rhs_
     }
     maplen = vapply_1i(mapunique, length)
-    idx = do.call("CJ", mapunique)[map, I := .I][["I"]] # TO DO: move this to C and avoid materialising the Cross Join.
+    idx = do.call("CJ", mapunique)[map, 'I' := .I][["I"]] # TO DO: move this to C and avoid materialising the Cross Join.
     ans = .Call(Cfcast, lhs, val, maplen[[1L]], maplen[[2L]], idx, fill, fill.default, is.null(fun.call))
     allcols = do.call("paste", c(rhs, sep=sep))
     if (length(valnames) > 1L)
@@ -220,26 +219,6 @@ dcast.data.table <- function(data, formula, fun.aggregate = NULL, sep = "_", ...
       # removed 'setcolorder()' here, #1153
     setattr(ans, 'names', c(lhsnames, allcols))
     setDT(ans); setattr(ans, 'sorted', lhsnames)
-  } else {
-    # formula is of the form x + y ~ . (rare case)
-    if (drop) {
-      if (is.null(subset) && is.null(fun.call)) {
-        dat = copy(dat) # can't be avoided
-        setkeyv(dat, lhsnames)
-      }
-      ans = dat
-    } else {
-      lhs = shallow(dat, lhsnames)
-      val = shallow(dat, valnames)
-      lhs_ = cj_uniq(lhs)
-      idx = lhs_[lhs, I := .I][["I"]]
-      lhs_[, I := NULL]
-      ans = .Call(Cfcast, lhs_, val, nrow(lhs_), 1L, idx, fill, fill.default, is.null(fun.call))
-      setDT(ans); setattr(ans, 'sorted', lhsnames)
-      setnames(ans, c(lhsnames, valnames))
-    }
-    if (length(valnames) == 1L)
-      setnames(ans, valnames, value.var)
-  }
+  } else stopf("Internal error -- empty rhsnames in dcast; please report") # nocov
   return (ans)
 }
