@@ -18,22 +18,23 @@ SEXP fcast(SEXP lhs, SEXP val, SEXP nrowArg, SEXP ncolArg, SEXP idxArg, SEXP fil
   for (int i=0; i<nval; ++i) {
     SEXP thiscol = VECTOR_ELT(val, i);
     SEXP thisfill = fill;
+    SEXPTYPE thistype = TYPEOF(thiscol);
     int nprotect = 0;
     if (isNull(fill)) {
       if (LOGICAL(is_agg)[0]) {
-        thisfill = PROTECT(allocNAVector(TYPEOF(thiscol), 1)); nprotect++;
+        thisfill = PROTECT(allocNAVector(thistype, 1)); nprotect++;
       } else thisfill = VECTOR_ELT(fill_d, i);
     }
-    if (TYPEOF(thisfill) != TYPEOF(thiscol)) {
-      thisfill = PROTECT(coerceVector(thisfill, TYPEOF(thiscol))); nprotect++;
+    if (TYPEOF(thisfill) != thistype) {
+      thisfill = PROTECT(coerceVector(thisfill, thistype)); nprotect++;
     }
-    switch (TYPEOF(thiscol)) {
+    switch (thistype) {
     case INTSXP:
     case LGLSXP: {
       const int *ithiscol = INTEGER(thiscol);
       const int *ithisfill = INTEGER(thisfill);
       for (int j=0; j<ncols; ++j) {
-        SET_VECTOR_ELT(ans, nlhs+j+i*ncols, target=allocVector(TYPEOF(thiscol), nrows) );
+        SET_VECTOR_ELT(ans, nlhs+j+i*ncols, target=allocVector(thistype, nrows) );
         int *itarget = INTEGER(target);
         copyMostAttrib(thiscol, target);
         for (int k=0; k<nrows; ++k) {
@@ -46,7 +47,7 @@ SEXP fcast(SEXP lhs, SEXP val, SEXP nrowArg, SEXP ncolArg, SEXP idxArg, SEXP fil
       const double *dthiscol = REAL(thiscol);
       const double *dthisfill = REAL(thisfill);
       for (int j=0; j<ncols; ++j) {
-        SET_VECTOR_ELT(ans, nlhs+j+i*ncols, target=allocVector(TYPEOF(thiscol), nrows) );
+        SET_VECTOR_ELT(ans, nlhs+j+i*ncols, target=allocVector(thistype, nrows) );
         double *dtarget = REAL(target);
         copyMostAttrib(thiscol, target);
         for (int k=0; k<nrows; ++k) {
@@ -55,9 +56,22 @@ SEXP fcast(SEXP lhs, SEXP val, SEXP nrowArg, SEXP ncolArg, SEXP idxArg, SEXP fil
         }
       }
     } break;
+    case CPLXSXP: {
+      const Rcomplex *zthiscol = COMPLEX(thiscol);
+      const Rcomplex *zthisfill = COMPLEX(thisfill);
+      for (int j=0; j<ncols; ++j) {
+        SET_VECTOR_ELT(ans, nlhs+j+i*ncols, target=allocVector(thistype, nrows) );
+        Rcomplex *ztarget = COMPLEX(target);
+        copyMostAttrib(thiscol, target);
+        for (int k=0; k<nrows; ++k) {
+          int thisidx = idx[k*ncols + j];
+          ztarget[k] = (thisidx == NA_INTEGER) ? zthisfill[0] : zthiscol[thisidx-1];
+        }
+      }
+    } break;
     case STRSXP:
       for (int j=0; j<ncols; ++j) {
-        SET_VECTOR_ELT(ans, nlhs+j+i*ncols, target=allocVector(TYPEOF(thiscol), nrows) );
+        SET_VECTOR_ELT(ans, nlhs+j+i*ncols, target=allocVector(thistype, nrows) );
         copyMostAttrib(thiscol, target);
         for (int k=0; k<nrows; ++k) {
           int thisidx = idx[k*ncols + j];
@@ -67,7 +81,7 @@ SEXP fcast(SEXP lhs, SEXP val, SEXP nrowArg, SEXP ncolArg, SEXP idxArg, SEXP fil
       break;
     case VECSXP:
       for (int j=0; j<ncols; ++j) {
-        SET_VECTOR_ELT(ans, nlhs+j+i*ncols, target=allocVector(TYPEOF(thiscol), nrows) );
+        SET_VECTOR_ELT(ans, nlhs+j+i*ncols, target=allocVector(thistype, nrows) );
         copyMostAttrib(thiscol, target);
         for (int k=0; k<nrows; ++k) {
           int thisidx = idx[k*ncols + j];
@@ -90,11 +104,10 @@ SEXP fcast(SEXP lhs, SEXP val, SEXP nrowArg, SEXP ncolArg, SEXP idxArg, SEXP fil
 // // # nocov start
 // // Note: all these functions below are internal functions and are designed specific to fcast.
 // SEXP zero_init(R_len_t n) {
-//   R_len_t i;
 //   SEXP ans;
 //   if (n < 0) error(_("Input argument 'n' to 'zero_init' must be >= 0"));
 //   ans = PROTECT(allocVector(INTSXP, n));
-//   for (i=0; i<n; i++) INTEGER(ans)[i] = 0;
+//   for (int i=0; i<n; ++i) INTEGER(ans)[i] = 0;
 //   UNPROTECT(1);
 //   return(ans);
 // }
@@ -126,11 +139,10 @@ SEXP fcast(SEXP lhs, SEXP val, SEXP nrowArg, SEXP ncolArg, SEXP idxArg, SEXP fil
 // }
 
 // SEXP diff_int(SEXP x, R_len_t n) {
-//   R_len_t i;
 //   SEXP ans;
 //   if (TYPEOF(x) != INTSXP) error(_("Argument 'x' to 'diff_int' must be an integer vector"));
 //   ans = PROTECT(allocVector(INTSXP, length(x)));
-//   for (i=1; i<length(x); i++)
+//   for (int i=1; i<length(x); ++i)
 //     INTEGER(ans)[i-1] = INTEGER(x)[i] - INTEGER(x)[i-1];
 //   INTEGER(ans)[length(x)-1] = n - INTEGER(x)[length(x)-1] + 1;
 //   UNPROTECT(1);
@@ -138,16 +150,16 @@ SEXP fcast(SEXP lhs, SEXP val, SEXP nrowArg, SEXP ncolArg, SEXP idxArg, SEXP fil
 // }
 
 // SEXP intrep(SEXP x, SEXP len) {
-//   R_len_t i,j,l=0, k=0;
+//   R_len_t l=0, k=0;
 //   SEXP ans;
 //   if (TYPEOF(x) != INTSXP || TYPEOF(len) != INTSXP) error(_("Arguments 'x' and 'len' to 'intrep' should both be integer vectors"));
 //   if (length(x) != length(len)) error(_("'x' and 'len' must be of same length"));
 //   // assuming both are of length >= 1
-//   for (i=0; i<length(len); i++)
+//   for (int i=0; i<length(len); ++i)
 //     l += INTEGER(len)[i]; // assuming positive values for len. internal use - can't bother to check.
 //   ans = PROTECT(allocVector(INTSXP, l));
-//   for (i=0; i<length(len); i++) {
-//     for (j=0; j<INTEGER(len)[i]; j++) {
+//   for (int i=0; i<length(len); ++i) {
+//     for (int j=0; j<INTEGER(len)[i]; ++j) {
 //       INTEGER(ans)[k++] = INTEGER(x)[i];
 //     }
 //   }
